package org.dbhelper.utils;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Savepoint;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Map;

/**
 *  ɶ, ִ  Statement closeִ DB ConnectionŬ
 * @author pigmon
 * Created on 2005. 12. 8.
 */
public class SmartConnection implements Connection {
	private Connection cn;
	private ArrayList<Statement> statements = null;

	public SmartConnection(Connection cn) {
		this.cn = cn;
		this.statements = new ArrayList<Statement>();
	}
	
	public SmartConnection() {
		this.statements = new ArrayList<Statement>();
	}
	
	public void setConnection(Connection cn) {
		this.cn = cn;
	}

	public Statement createStatement() throws SQLException {
		Statement stmt = cn.createStatement();
		statements.add(stmt);
		return stmt;
	}

	public PreparedStatement prepareStatement(String sql) throws SQLException {
		PreparedStatement stmt = cn.prepareStatement(sql);
		statements.add(stmt);
		return stmt;
	}

	public CallableStatement prepareCall(String sql) throws SQLException {
		CallableStatement stmt = cn.prepareCall(sql);
		statements.add(stmt);
		return stmt;
	}

	public String nativeSQL(String sql) throws SQLException {
		return cn.nativeSQL(sql);
	}

	public void setAutoCommit(boolean autoCommit) throws SQLException {
		cn.setAutoCommit(autoCommit);

	}

	public boolean getAutoCommit() throws SQLException {
		return cn.getAutoCommit();
	}

	public void commit() throws SQLException {
		cn.commit();
	}

	public void rollback() throws SQLException {
		cn.rollback();

	}

	public void close() throws SQLException {
		for (Statement s : statements) {
			try {
				s.close();
			}
			catch (SQLException e) {
			}
		}
		cn.close();
	}

	public boolean isClosed() throws SQLException {
		return cn.isClosed();
	}

	public DatabaseMetaData getMetaData() throws SQLException {
		return cn.getMetaData();
	}

	public void setReadOnly(boolean readOnly) throws SQLException {
		cn.setReadOnly(readOnly);

	}

	public boolean isReadOnly() throws SQLException {
		return cn.isReadOnly();
	}

	public void setCatalog(String catalog) throws SQLException {
		cn.setCatalog(catalog);

	}

	public String getCatalog() throws SQLException {
		return cn.getCatalog();
	}

	public void setTransactionIsolation(int level) throws SQLException {
		cn.setTransactionIsolation(level);

	}

	public int getTransactionIsolation() throws SQLException {
		return cn.getTransactionIsolation();
	}

	public SQLWarning getWarnings() throws SQLException {
		return cn.getWarnings();
	}

	public void clearWarnings() throws SQLException {
		cn.clearWarnings();

	}

	public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
		Statement stmt = cn.createStatement(resultSetType, resultSetConcurrency);
		statements.add(stmt);
		return stmt;
	}

	public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency)
			throws SQLException {
		PreparedStatement stmt = cn.prepareStatement(sql, resultSetType, resultSetConcurrency);
		statements.add(stmt);
		return stmt;
	}

	public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency) throws SQLException {
		CallableStatement stmt = cn.prepareCall(sql, resultSetType, resultSetConcurrency);
		statements.add(stmt);
		return stmt;
	}

	public Map<String, Class<?>> getTypeMap() throws SQLException {
		return cn.getTypeMap();
	}

	public void setTypeMap(Map<String, Class<?>> arg0) throws SQLException {
		cn.setTypeMap(arg0);

	}

	public void setHoldability(int holdability) throws SQLException {
		cn.setHoldability(holdability);
	}

	public int getHoldability() throws SQLException {
		return cn.getHoldability();
	}

	public Savepoint setSavepoint() throws SQLException {
		return cn.setSavepoint();
	}

	public Savepoint setSavepoint(String name) throws SQLException {
		return cn.setSavepoint(name);
	}

	public void rollback(Savepoint savepoint) throws SQLException {
		cn.rollback(savepoint);

	}

	public void releaseSavepoint(Savepoint savepoint) throws SQLException {
		cn.releaseSavepoint(savepoint);

	}

	public Statement createStatement(int resultSetType, int resultSetConcurrency, int resultSetHoldability)
			throws SQLException {
		Statement stmt = cn.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability);
		statements.add(stmt);
		return stmt;
	}

	public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency,
			int resultSetHoldability) throws SQLException {
		PreparedStatement stmt = cn.prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
		statements.add(stmt);
		return stmt;
	}

	public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency,
			int resultSetHoldability) throws SQLException {
		CallableStatement stmt = cn.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
		statements.add(stmt);
		return stmt;
	}

	public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
		PreparedStatement stmt = cn.prepareStatement(sql, autoGeneratedKeys);
		statements.add(stmt);
		return stmt;
	}

	public PreparedStatement prepareStatement(String sql, int[] columnIndexes) throws SQLException {
		PreparedStatement stmt = cn.prepareStatement(sql, columnIndexes);
		statements.add(stmt);
		return stmt;
	}

	public PreparedStatement prepareStatement(String sql, String[] columnNames) throws SQLException {
		PreparedStatement stmt = cn.prepareStatement(sql, columnNames);
		statements.add(stmt);
		return stmt;
	}
}
